/**
 * 系统中使用的工具
 * 包括 草稿箱id生成规则
 * 包括 密码加密规则
 */

import moment = require("moment");
import { wenZhangQueue } from "../biz/task";
const md5 = require("md5");
const provinces = require('province-city-china/dist/data.json');

/**
 * 返回编辑正确的结果
 * @returns 
 */
export function successResult() {
    return {success:true}
}

/**
 * 生成系统id
 * @param tableName 
 * @param userId 
 * @returns 
 */
export function generateSystemId(tableName:string, userId:string) {
    return md5(`${tableName}${Math.floor(Math.random() * 10000)}${userId}${new Date().valueOf()}${Math.floor(Math.random() * 100)}`);
}

/**
 * 生成token
 * @param userId 
 * @returns 
 */
export function generateToken(userId:string) {
    return md5(`${userId}${Math.floor(Math.random() * 10000)}${new Date().valueOf()}${Math.floor(Math.random() * 100)}`);
}

/**
 * 生成userid
 */
export function generateUserId() {
    return md5(`${Math.random() * 100}${new Date().valueOf()}${Math.floor(Math.random() * 10000)}`);
}

/**
 * 获取code的id
 * @param loginId 用户Id
 * @param todaySendCount 今日发送次数
 * @returns ''
 */
 export function getSMSCodeId(loginId:string, todaySendCount:number) {
    return md5(`${loginId}${todaySendCount}${new Date().valueOf()}`);
}


/**
 * 获取一个随机6位数的验证码
 * @returns 
 */
export function generateSMSCode() {
    let code = ``;
    for (let i =0; i < 6; i++) {
        code += Math.floor(Math.random() * 10);
    }
    return code;
}


/**
 * 生成唯一订单id
 */
export function generateOrderId(userId:string) {
    return md5(`${userId}${Math.random() * 100}${new Date().valueOf()}${Math.floor(Math.random() * 1000)}`);
}


/*
 * 获取今天开始时刻的时间戳  0时0分
 * @returns 
 */
export function getTodayMs() {
    return new Date(`${ moment().format("YYYY-MM-DD")} 00:00:00`).valueOf();
}



export function generateWXOrderId(memberType, userId) {
    let thisDate = new Date();
    return `${md5(`${memberType}${userId}${Math.random() * 100}${thisDate.valueOf()}${Math.floor(Math.random() * 1000)}`)}`
}



/**
 * 获取届次
 * @param year 入会年份 
 * @param month 入会月份
 * 下拉选择起始届次：2008年第四届第一次
 * @returns 
 */
export function getEdition(year, month) {
    //起始年份和月份 第一届第一次
    const startYear = 1993;
    const Num = ((year-startYear)+1)/5;
    let jie = Math.ceil(Num);
    let ci = Math.ceil(((Num*100) - ((jie - 1)*100))/10);
    if (month <= 6)  ci = ci - 1;

    return `第${jie}届,第${ci}次`;
    // const startYear = 1993;
    // let jie = Math.ceil(((year-startYear)+1)/5);
    // let ci = ((((year-startYear)+1)/5) - (jie - 1)) * 10;
    // if (month < 6)  ci = ci -1;


    // let currentYear = new Date().getFullYear();
    // if (year > currentYear) throw new BizError(ERRORENUM.无法获取未来届次);

    // //计算从起始时间给定时间的总月份数
    // let totalMonth = (year - startYear) * 12 + month;
    // //每次是6个月
    // let sessions = Math.ceil(totalMonth / 6);
    // //每10次一届，从第四届开始计算
    // let times = Math.ceil(sessions / 10);
    // let sessionNumber = sessions - (times - 1) * 10 //届次是从第四届第一次开始，所以需要减一

    // return `第${times}届,第${sessionNumber}次`;
}


export function changeArticleName(idList) {
    let str = '';
    idList.forEach((id, index) => {
        if (index) str += ',';
        str += `${wenZhangQueue[id]}`;
    });
    return str;
}


/**
 * 获取行政区域代码
 * @param areaName 
 */
export function getAreaCode(areaName) {
    let areaCode;
    provinces.forEach( info => {
        let {code, name, province} = info; //城市代码、城市名称、省/直辖市/特别行政区代码
        if (name == areaName) areaCode = province
    })
    
    return areaCode;
}


/**
 * 获取会员编号
 * @param db 
 * @param memberTypeStr 
 * @param joinTime 
 * @param sheng 
 * @returns CEFA+入会时间+行政区域代码+5位流水号+DW（单位）/GR（个人）
 * 个人会员：以审核时间算正式入会时间
 * 单位会员：以审核时间算正式入会时间
 * 例 CEFA-20230620-11-00001GR
 */
export async function getMenberNum(db, joinTime, sheng, memberTypeStr) {
    //定义编号前缀
    const prefix = 'CEFA';

    //获取流水号计数器
    const counterCollection = db.collection('serialNumberCounter');

    // 尝试找到并更新计数器  
    let result = await counterCollection.findOneAndUpdate(  
        { _id: 'memberCounter' },  
        { $inc: { count: 1 } },  
        { upsert: true, returnDocument: 'after' }  
    );
    // 生成5位流水号（这里简单地将计数器值转为字符串并填充0至5位）  
    let serialNumber = (result.value.count + '').padStart(5, '0');

    let areaCode = getAreaCode(sheng);
    let memberNum = `${prefix}-${moment(joinTime).format("YYYYMMDD")}-${areaCode}-${serialNumber}${memberTypeStr}`
    
    return memberNum;
}


